using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO;
using System.Text;
using System.Text.RegularExpressions;

namespace PatchInternalVisibleTo {
	class Program {
		static void Main(string[] args) {
			Console.WriteLine("InternalVisibleTo attribute patcher (C) Developer Express Inc.");
			if (args.Length != 3) {
				Console.WriteLine("Usage: PatchInternalVisibleTo <patched file path> <sn.exe full path> <strong key full path>");
				return;
			}

			string fileName = args[0];
			if (!File.Exists(fileName)) {
				Console.WriteLine("The file {0} not found", fileName);
				return;
			}
			string snExe = args[1];
			if (!File.Exists(fileName)) {
				Console.WriteLine("The file {0} not found", snExe);
				return;
			}
			string keyFileName = args[2];
			if (!File.Exists(keyFileName)) {
				Console.WriteLine("The file {0} not found", keyFileName);
				return;
			}

			PerformPatch(fileName, snExe, keyFileName);
		}

		static void PerformPatch(string fileName, string snExe, string keyFileName) {
			string publicKeyToken = CreatePublicKeyToken(snExe, keyFileName);
			PatchFile(fileName, publicKeyToken);
		}

		static void PatchFile(string fileName, string publicKeyToken) {
			string content = String.Empty;
			using (StreamReader reader = new StreamReader(fileName)) {
				content = reader.ReadToEnd();
				reader.Close();
			}

			Regex regex = new Regex(@""",\s*PublicKey=[0123456789abcdefABCDEF]*""");
			content = regex.Replace(content, String.Format("\", PublicKey={0}\"", publicKeyToken));

			string tmpFileName = Path.GetRandomFileName();
			using (StreamWriter writer = new StreamWriter(tmpFileName)) {
				writer.Write(content);
				writer.Close();
			}
			File.Delete(fileName);
			File.Move(tmpFileName, fileName);
		}

		static string CreatePublicKeyToken(string snExe, string keyFileName) {
			string publicKeyName = CreatePublicKey(snExe, keyFileName);
			string publicKeyToken = ObtainPublicKeyToken(snExe, publicKeyName);
			File.Delete(publicKeyName);
			return publicKeyToken;
		}

		static string CreatePublicKey(string snExe, string keyFileName) {
			const string fileName = "dx_public_key.tmp";
			ProcessStartInfo psi = new ProcessStartInfo();
			psi.FileName = snExe;
			psi.Arguments = String.Format("-q -p {0} {1}", keyFileName, fileName);
			psi.UseShellExecute = false;
			//psi.CreateNoWindow = true;
			Process proc = Process.Start(psi);
			proc.WaitForExit();
			return fileName;
		}

		static string ObtainPublicKeyToken(string snExe, string publicKeyName) {
			const string fileName = "dx_public_key_token.csv.tmp";
			ProcessStartInfo psi = new ProcessStartInfo();
			psi.FileName = snExe;
			psi.Arguments = String.Format("-q -o {0} {1}", publicKeyName, fileName);
			//psi.CreateNoWindow = true;
			psi.UseShellExecute = false;

			Process proc = Process.Start(psi);
			proc.WaitForExit();

			string csvContent = String.Empty;
			using (StreamReader sr = new StreamReader(fileName)) {
				csvContent = sr.ReadToEnd();
				sr.Close();
			}
			File.Delete(fileName);

			string[] bytes = csvContent.Split(',');
			StringBuilder result = new StringBuilder();
			int count = bytes.Length;
			for (int i = 0; i < count; i++) {
				int byteValue = Int32.Parse(bytes[i]);
				result.AppendFormat("{0:x2}", byteValue);
			}
			return result.ToString();
		}
	}
}
